/*
  ******************************************************************************
  * @file    user.c 
  * @author  AMG - RF Application Team
  * @version V1.0.0
  * @date    12 - 10 - 2017
  * @brief   Application functions
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2017 STMicroelectronics</center></h2>
  ******************************************************************************
  */ 
  
/** \cond DOXYGEN_SHOULD_SKIP_THIS
 */ 
 
/* Includes-----------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "BlueNRG1_it.h"
#include "BlueNRG1_conf.h"
#include "ble_const.h"
#include "bluenrg1_stack.h"
#include "gp_timer.h"
#include "SDK_EVAL_Config.h"
#include "OTA_btl.h"
#include "user.h"
#include "osal.h"

#ifndef DEBUG
#define DEBUG 0
#endif

#if DEBUG
#include <stdio.h>
#define PRINTF(...) printf(__VA_ARGS__)
#else
#define PRINTF(...)
#endif


/******************************************************************************
 * Function Name  : device_initialization.
 * Description    : Init a BlueNRG device.
 * Input          : None.
 * Output         : None.
 * Return         : None.
******************************************************************************/
void device_initialization(void)
{
  uint16_t service_handle;
  uint16_t dev_name_char_handle;
  uint16_t appearance_char_handle;
  uint8_t status;

  uint8_t value_2[] = {0xAA,0x9A,0x78,0x56,0x34,0x12};

  //aci_hal_write_config_data
  //status = aci_hal_write_config_data(offset,length,value);
  status = aci_hal_write_config_data(0x00,0x06,value_2);
  if (status != BLE_STATUS_SUCCESS) {
    PRINTF("aci_hal_write_config_data() failed:0x%02x\r\n", status);
  }else{
    PRINTF("aci_hal_write_config_data --> SUCCESS\r\n");
  }

  //aci_gatt_init
  //status = aci_gatt_init();
  status = aci_gatt_init();
  if (status != BLE_STATUS_SUCCESS) {
    PRINTF("aci_gatt_init() failed:0x%02x\r\n", status);
  }else{
    PRINTF("aci_gatt_init --> SUCCESS\r\n");
  }

  //aci_gap_init
  //status = aci_gap_init(role,privacy_enabled,device_name_char_len, &service_handle, &dev_name_char_handle, &appearance_char_handle);
  status = aci_gap_init(GAP_PERIPHERAL_ROLE,0x00,0x07, &service_handle, &dev_name_char_handle, &appearance_char_handle);
  if (status != BLE_STATUS_SUCCESS) {
    PRINTF("aci_gap_init() failed:0x%02x\r\n", status);
  }else{
    PRINTF("aci_gap_init --> SUCCESS\r\n");
  }

  //aci_hal_set_tx_power_level
  //status = aci_hal_set_tx_power_level(en_high_power,pa_level);
  status = aci_hal_set_tx_power_level(0x01,0x04);
  if (status != BLE_STATUS_SUCCESS) {
    PRINTF("aci_hal_set_tx_power_level() failed:0x%02x\r\n", status);
  }else{
    PRINTF("aci_hal_set_tx_power_level --> SUCCESS\r\n");
  }
}


/******************************************************************************
 * Function Name  : set_device_discoverable.
 * Description    : Puts the device in connectable mode.
 * Input          : None.
 * Output         : None.
 * Return         : None.
******************************************************************************/
void set_device_discoverable(void)
{
  uint8_t status;

  //aci_gap_set_discoverable
  //status = aci_gap_set_discoverable(advertising_type,advertising_interval_min,advertising_interval_max,own_address_type,advertising_filter_policy,local_name_length,local_name_length,service_uuid_length,service_uuid_length,slave_conn_interval_min,slave_conn_interval_max);
  status = aci_gap_set_discoverable(ADV_NONCONN_IND,0x00A0,0x00A0,0x00,NO_WHITE_LIST_USE,0x00,NULL,0x00,NULL,0x0000,0x0000);
  if (status != BLE_STATUS_SUCCESS) {
    PRINTF("aci_gap_set_discoverable() failed:0x%02x\r\n", status);
  }else{
    PRINTF("aci_gap_set_discoverable --> SUCCESS\r\n");
  }

  uint8_t advertising_data_12[] = {0x02,0x01,0x06,0x1A,0xFF,0x30,0x00,0x02,0x15,0xE2,0x0A,0x39,0xF4,0x73,0xF5,0x4B,0xC4,0xA1,0x2F,0x17,0xD1,0xAD,0x07,0xA9,0x61,0x00,0x01,0x00,0x02,0xC8,0x00};

  //hci_le_set_advertising_data
  //status = hci_le_set_advertising_data(advertising_data_length,advertising_data);
  status = hci_le_set_advertising_data(0x1E,advertising_data_12);
  if (status != BLE_STATUS_SUCCESS) {
    PRINTF("hci_le_set_advertising_data() failed:0x%02x\r\n", status);
  }else{
    PRINTF("hci_le_set_advertising_data --> SUCCESS\r\n");
  }
}


/******************************************************************************
 * Function Name  : app_tick.
 * Description    : Device Demo state machine.
 * Input          : None.
 * Output         : None.
 * Return         : None.
******************************************************************************/
void APP_Tick(void)
{

 /* User has to add his code */
}
/* *************** BlueNRG-1 Stack Callbacks****************/




/** \endcond 
*/
